@@ -11,8 +11,10 @@ from torch import Tensor
 from typing import Any
 from typing import Dict
 from typing import List
+from typing import Type
 from typing import Tuple
 from typing import Union
+from typing import TypeVar
 from typing import Callable
 from typing import Optional
 from typing import NamedTuple
@@ -36,6 +38,7 @@ from ....constants import INPUT_KEY
 from ....misc.toolkit import slerp
 from ....misc.toolkit import new_seed
 from ....misc.toolkit import eval_context
+from ....misc.toolkit import download_model
 from ....misc.toolkit import seed_everything
 from ....modules.blocks import Conv2d
 from ....models.cv.diffusion import LDM
@@ -88,6 +91,9 @@ class MaskedCond(NamedTuple):
     remained_image_cond: Tensor
 
 
+T = TypeVar("T", bound="DiffusionAPI")
+
+
 class DiffusionAPI(APIMixin):
     m: DDPM
     sampler: ISampler
@@ -868,56 +874,56 @@ class DiffusionAPI(APIMixin):
 
     @classmethod
     def from_sd(
-        cls,
+        cls: Type[T],
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
         use_half: bool = False,
-    ) -> "DiffusionAPI":
+    ) -> T:
         return cls.from_pipeline(ldm_sd(), device, use_amp=use_amp, use_half=use_half)
 
     @classmethod
     def from_sd_version(
-        cls,
+        cls: Type[T],
         version: str,
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
         use_half: bool = False,
-    ) -> "DiffusionAPI":
+    ) -> T:
         m = ldm_sd_version(version)
         return cls.from_pipeline(m, device, use_amp=use_amp, use_half=use_half)
 
     @classmethod
     def from_sd_anime(
-        cls,
+        cls: Type[T],
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
         use_half: bool = False,
-    ) -> "DiffusionAPI":
+    ) -> T:
         m = ldm_sd_anime()
         return cls.from_pipeline(m, device, use_amp=use_amp, use_half=use_half)
 
     @classmethod
     def from_sd_inpainting(
-        cls,
+        cls: Type[T],
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
         use_half: bool = False,
-    ) -> "DiffusionAPI":
+    ) -> T:
         m = ldm_sd_inpainting()
         return cls.from_pipeline(m, device, use_amp=use_amp, use_half=use_half)
 
     @classmethod
     def from_sd_v2(
-        cls,
+        cls: Type[T],
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
         use_half: bool = False,
-    ) -> "DiffusionAPI":
+    ) -> T:
         return cls.from_pipeline(
             ldm_sd_v2(),
             device,
@@ -928,12 +934,12 @@ class DiffusionAPI(APIMixin):
 
     @classmethod
     def from_sd_v2_base(
-        cls,
+        cls: Type[T],
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
         use_half: bool = False,
-    ) -> "DiffusionAPI":
+    ) -> T:
         return cls.from_pipeline(
             ldm_sd_v2_base(),
             device,
@@ -944,44 +950,44 @@ class DiffusionAPI(APIMixin):
 
     @classmethod
     def from_celeba_hq(
-        cls,
+        cls: Type[T],
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
         use_half: bool = False,
-    ) -> "DiffusionAPI":
+    ) -> T:
         m = ldm_celeba_hq()
         return cls.from_pipeline(m, device, use_amp=use_amp, use_half=use_half)
 
     @classmethod
     def from_inpainting(
-        cls,
+        cls: Type[T],
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
         use_half: bool = False,
-    ) -> "DiffusionAPI":
+    ) -> T:
         m = ldm_inpainting()
         return cls.from_pipeline(m, device, use_amp=use_amp, use_half=use_half)
 
     @classmethod
     def from_sr(
-        cls,
+        cls: Type[T],
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
         use_half: bool = False,
-    ) -> "DiffusionAPI":
+    ) -> T:
         return cls.from_pipeline(ldm_sr(), device, use_amp=use_amp, use_half=use_half)
 
     @classmethod
     def from_semantic(
-        cls,
+        cls: Type[T],
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
         use_half: bool = False,
-    ) -> "DiffusionAPI":
+    ) -> T:
         m = ldm_semantic()
         return cls.from_pipeline(m, device, use_amp=use_amp, use_half=use_half)
 
@@ -1089,6 +1095,63 @@ class DiffusionAPI(APIMixin):
             )
 
 
+class ControlledDiffusionAPI(DiffusionAPI):
+    current: Optional[str]
+    weights: tensor_dict_type
+
+    def __init__(
+        self,
+        m: DDPM,
+        device: torch.device,
+        *,
+        use_amp: bool = False,
+        use_half: bool = False,
+        clip_skip: int = 0,
+        hint_channels: int = 3,
+    ):
+        super().__init__(
+            m,
+            device,
+            use_amp=use_amp,
+            use_half=use_half,
+            clip_skip=clip_skip,
+        )
+        self.m.make_control_net(hint_channels)
+        self.current = None
+        self.weights: Dict[str, tensor_dict_type] = {}
+
+    def to(
+        self,
+        device: torch.device,
+        *,
+        use_amp: bool = False,
+        use_half: bool = False,
+    ) -> None:
+        super().to(device, use_amp=use_amp, use_half=use_half)
+        for d in self.weights.values():
+            for k, v in d.items():
+                v = v.half() if use_half else v.float()
+                d[k] = v.to(device)
+
+    def prepare(self, names2tags: Dict[str, str]) -> None:
+        for name, tag in names2tags.items():
+            self.weights[name] = torch.load(download_model(tag))
+
+    def switch(self, name: str) -> None:
+        if self.m.control_model is None:
+            raise ValueError("`control_model` is not built yet")
+        if name == self.current:
+            return
+        d = self.weights.get(name)
+        if d is None:
+            raise ValueError(
+                f"cannot find weights called '{name}', "
+                f"available weights are: {', '.join(self.weights.keys())}"
+            )
+        self.current = name
+        self.m.control_model.load_state_dict(d)
+
+
 def _ldm(
     model: str,
     latent_size: int,
@@ -1253,4 +1316,5 @@ def ldm_semantic(pretrained: bool = True) -> DLPipeline:
 
 __all__ = [
     "DiffusionAPI",
+    "ControlledDiffusionAPI",
 ]
