@@ -8,14 +8,14 @@ from typing import Callable
 from typing import Optional
 from cftool.array import to_torch
 
-from ...protocol import IImageExtractor
-from ...protocol import ImageFolderLatentResponse
+from ...schema import IImageExtractor
+from ...schema import ImageFolderLatentResponse
 from ...cv.utils import predict_paths
 from ...cv.utils import predict_folder
 from ...cv.pipeline import CVPipeline
 from ....data import TensorData
 from ....types import texts_type
-from ....protocol import IInference
+from ....schema import IInference
 from ....constants import INPUT_KEY
 from ....constants import LATENT_KEY
 from ....misc.toolkit import eval_context
