@@ -1,6 +1,6 @@
+from .schema import *
 from .attn import *
 from .vqgan import *
 from .vanilla import *
-from .protocol import *
 from .style_gan import *
 from .style_gan_v2 import *
