@@ -1,10 +1,10 @@
+from .schema import *
 from .attn import *
 from .fnet import *
 from .mixer import *
 from .vqgan import *
 from .vanilla import *
 from .backbone import *
-from .protocol import *
 from .perceiver import *
 from .pool_former import *
 from .transformer import *
